package com.gree.mall.contest.controller.mini.goods;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.goods.GoodsFavoriteBean;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.goods.GoodsFavoriteLogic;
import com.gree.mall.contest.plus.entity.GoodsFavorite;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@Slf4j
@RestController
@Tag(name = "商品收藏", description = "商品收藏")
@RequestMapping(value = "/mini/goods/favorite", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class MiniGoodsFavoriteController {

    private final GoodsFavoriteLogic goodsFavoriteLogic;

    @PostMapping("/add")
    @Operation(summary = "添加收藏")
    public ResponseHelper<GoodsFavorite> add(
            HttpServletRequest request,
            @Parameter(description = "用户id", required = true) @RequestParam(required = true) String userId,
            @Parameter(description = "商品id", required = true) @RequestParam(required = true) String goodsId,
            @Parameter(description = "商品规格id", required = false) @RequestParam(required = false) String goodsSpecId
    ) throws Exception {
        GoodsFavorite goodsFavorite = goodsFavoriteLogic.saveGoodsFavorite(request, userId, goodsId, goodsSpecId);
        return ResponseHelper.success(goodsFavorite);
    }

    @PostMapping("/del")
    @Operation(summary = "删除收藏")
    public ResponseHelper<GoodsFavorite> del(
            @Parameter(description = "商品收藏id", required = true) @RequestParam(required = true) String goodFavoriteId

    ) throws Exception {
        goodsFavoriteLogic.delGoodsFavorite(goodFavoriteId);
        return ResponseHelper.success();
    }

    @PostMapping("/detail/del")
    @Operation(summary = "从商品详情页面删除收藏")
    public ResponseHelper<GoodsFavorite> detailDel(
            @Parameter(description = "商品id", required = true) @RequestParam(required = true) String goodsId,
            @Parameter(description = "用户id", required = true) @RequestParam(required = true) String userId,
            @Parameter(description = "商品规格id", required = false) @RequestParam(required = false) String goodSpecId

    ) throws Exception {
        goodsFavoriteLogic.detailDelGoodsFavorite(goodsId, userId, goodSpecId);
        return ResponseHelper.success();
    }

    @GetMapping("/query")
    @Operation(summary = "获取商品收藏列表")
    public ResponseHelper<Page<GoodsFavoriteBean>> queryList(
            @Parameter(description = "用户id", required = true) @RequestParam(required = true) String userId,
            @Parameter(description = "页号", required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam Integer pageSize
    ) throws Exception {
        IPage<GoodsFavoriteBean> page = goodsFavoriteLogic.pageGoodsFavorite(userId, pageNum, pageSize);
        return ResponseHelper.success(page);
    }


}
