package com.gree.mall.contest.controller.mini.order;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.bean.goods.GoodsComment;
import com.gree.mall.contest.bean.goods.GoodsCommentCount;
import com.gree.mall.contest.bean.order.OrderCommentBean;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.order.OrderCommentLogic;
import com.gree.mall.contest.plus.entity.CommentTag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@Slf4j
@RestController
@Tag(name = "订单评价", description = "订单评价")
@RequestMapping(value = "/mini/order/comment", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class MiniOrderCommentController {

    private final OrderCommentLogic orderCommentLogic;

    @PostMapping("/add")
    @Operation(summary = "评价")
    public ResponseHelper add(
            HttpServletRequest request,
            @RequestBody OrderCommentBean orderCommentBean
    ) throws RemoteServiceException {
        orderCommentLogic.add(request , orderCommentBean);
        return ResponseHelper.success();
    }

    @GetMapping("/tag/list")
    @Operation(summary = "评价标签列表")
    public ResponseHelper<List<CommentTag>> tagList(HttpServletRequest request) throws RemoteServiceException {
        List<CommentTag> list = orderCommentLogic.list(request);
        return ResponseHelper.success(list);
    }


    @GetMapping("/goods")
    @Operation(summary = "商品的评价列表")
    public ResponseHelper<IPage<GoodsComment>> goods(
            HttpServletRequest request,
            @Parameter(description = "商品id",required = true) @RequestParam(required = true) String goodsId,
            @Parameter(description = "标签名",required = false) @RequestParam(required = false) String tag,
            @Parameter(description = "页号",required = true) @RequestParam(required = true) Integer pageNo,
            @Parameter(description = "页大小",required = true) @RequestParam(required = true) Integer pageSize
    ) throws RemoteServiceException {
        IPage<GoodsComment> goodsCommentIPage = orderCommentLogic.goodsCommentList(request , goodsId, tag, pageNo, pageSize);
        return ResponseHelper.success(goodsCommentIPage);
    }

    @GetMapping("/goods/count")
    @Operation(summary = "商品的评价汇总")
    public ResponseHelper<GoodsCommentCount> goodsCount(
            HttpServletRequest request,
            @Parameter(description = "商品id",required = true) @RequestParam(required = true) String goodsId
    ) throws RemoteServiceException {
        GoodsCommentCount goodsCommentCount = orderCommentLogic.goodsCommentCount(request, goodsId);
        return ResponseHelper.success(goodsCommentCount);
    }


}
