package com.gree.mall.contest.controller.pc.common;

import com.google.common.collect.Lists;
import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.SMSLogic;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.logic.common.WechatLogic;
import com.gree.mall.contest.plus.entity.AdminCompanyWechat;
import com.gree.mall.contest.plus.entity.CommonFile;
import com.gree.mall.contest.plus.entity.Region;
import com.gree.mall.contest.plus.service.AdminCompanyWechatService;
import com.gree.mall.contest.utils.CommonUtils;
import com.gree.mall.contest.utils.RedisUtil;
import com.gree.mall.contest.utils.VerifiUtils;
import com.gree.mall.contest.utils.oss.OSSUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;
import java.util.UUID;


@Slf4j
@RestController
@Tag(name = "公共API", description = "公共API")
@RequestMapping(value = "/pc/common", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class CommonController {

    private final CommonLogic commonLogic;
    private final WechatLogic wechatLogic;
    private final AdminCompanyWechatService adminCompanyWechatService;
    private final OSSUtil ossUtil;
    private final SMSLogic smsLogic;
    private final RedisUtil redisUtil;

    @PostMapping("/upload")
    @Operation(summary = "文件上传")
    public ResponseHelper<CommonFile> upload(
            @Parameter(required = true, description = "附件") @RequestParam(required = true) MultipartFile file
    ) throws IOException, RemoteServiceException {
        CommonFile commonFile = commonLogic.uploadFile(file);
        return ResponseHelper.success(commonFile);
    }


//    @ApiNotAuth
//    @GetMapping("/region/list")
//    @Operation(summary = "地区列表")
//    public ResponseHelper<List<Region>> regionList(
//            @Parameter(description = "父id(第一级传0)",required = true) @RequestParam(required = true) String pid
//    ) throws RemoteServiceException {
//        List<Region> list = commonLogic.queryRegionList(Lists.newArrayList(pid));
//        return ResponseHelper.success(list);
//    }
//
//
//    @PostMapping("/region/list2")
//    @Operation(summary = "地区列表2")
//    public ResponseHelper<List<Region>> regionList(
//            @RequestBody(required = true) List<String> pid
//    ) throws RemoteServiceException {
//        List<Region> list = commonLogic.queryRegionList(pid);
//        return ResponseHelper.success(list);
//    }


    @PostMapping("/companywechat/get")
    @Operation(summary = "获取商户配置二维码头像等详情")
    public ResponseHelper<AdminCompanyWechat> getCompanyWechat() {
        AdminCompanyWechat one = adminCompanyWechatService.lambdaQuery()
                .eq(AdminCompanyWechat::getCompanyWechatId, "1").one();
        return ResponseHelper.success(one);
    }

    @GetMapping("template/download")
    @Operation(summary = "下载模板")
    public void download(@RequestParam(required = false) @NotBlank(message = "需下载的模板名称不能为空") String name, HttpServletResponse response) throws IOException {
        CommonUtils.downloadFile(name, response);
    }

    @ApiNotAuth
    @Operation(summary = "获取获取前往小程序链接")
    @GetMapping("/getUrlScheme")
    public ResponseHelper<String> getUrlScheme(
            @Parameter(description = "通过 scheme 码进入小程序时的 query", required = false) @RequestParam(required = false) String query,
            @Parameter(required = false) @RequestParam(required = false) String companyWechatId
    ) throws IOException, WxErrorException {
        return ResponseHelper.success(wechatLogic.getUrlScheme(query, companyWechatId));
    }

    @ApiNotAuth
    @GetMapping("/img/get")
    @Operation(summary = "获取图片")
    public void getImg(
            @Parameter(required = true, description = "附件key") @RequestParam(required = true) String key,
            HttpServletResponse response
    ) throws IOException, RemoteServiceException {
        String file = commonLogic.getFile(key);
        response.sendRedirect(file);
    }


    @ApiNotAuth
    @GetMapping("/file/get")
    @Operation(summary = "获取文件")
    public ResponseHelper<String> getFile(
            @Parameter(required = true, description = "附件key") @RequestParam(required = true) String key,
            HttpServletResponse response
    ) throws IOException, RemoteServiceException {
        String filepath = ossUtil.getUrlWw(key);
        return ResponseHelper.success(filepath);
    }


    @ApiNotAuth
    @GetMapping("oss/config")
    @Operation(summary = "获取oss配置")
    public ResponseHelper<Map<String, String>> getOssConfig() throws UnsupportedEncodingException {
        return ResponseHelper.success(commonLogic.getOSSConfig());
    }


    @GetMapping("/getVerifi")
    @Operation(summary = "获取拖拽式验证码")
    public ResponseHelper<Map<String, Object>> getVerifi() {
        Map<String, Object> map = VerifiUtils.getVerifi();
        Integer xWidth = (Integer) map.get("xWidth");
        String key = UUID.randomUUID().toString();
        redisUtil.set(Constant.RedisPrefix.VERIFICATION + ":" + key, xWidth, 2 * 60);
        map.remove("xWidth");
        map.put("key", key);
        return ResponseHelper.success(map);
    }


    @ApiNotAuth
    @PostMapping("/sms/send")
    @Operation(summary = "发送验证码（绑定手机号专用）")
    public ResponseHelper sendSms(
            @Parameter(description = "手机号码", required = true) @RequestParam String phone,
            @Parameter(description = "滑动验证key", required = false) @RequestParam(required = false) String key,
            @Parameter(description = "滑动验证数值", required = false) @RequestParam(required = false) String vrifyCode,
            @Parameter(description = "发送类型 BIND=绑定手机号 RESET=找回密码", required = false) @RequestParam(required = false, defaultValue = "BIND") String sendType
    ) throws Exception {

        smsLogic.sendSms(phone, key, vrifyCode, sendType);
        return ResponseHelper.success();
    }

    @ApiNotAuth
    @GetMapping("/region/list")
    @Operation(summary = "地区列表")
    public ResponseHelper<List<Region>> regionList(
            @Parameter(description = "父id(第一级传0)", required = true) @RequestParam String pid
    ) throws RemoteServiceException {
        List<Region> list = commonLogic.queryRegionList(Lists.newArrayList(pid));
        return ResponseHelper.success(list);
    }

}
