package com.gree.mall.contest.controller.pc.goods;

import com.gree.mall.contest.bean.goods.GoodsCategoryBean;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.goods.GoodsCategoryLogic;
import com.gree.mall.contest.plus.entity.GoodsCategory;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Slf4j
@RestController
@Tag(name = "分类管理", description = "分类管理")
@RequestMapping(value = "/pc/goods/category", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class GoodsCategoryController {

    private final GoodsCategoryLogic goodsCategoryLogic;

    @GetMapping("/list")
    @Operation(summary = "分类管理列表")
    public ResponseHelper<List<GoodsCategoryBean>> page(
            @Parameter(description = "层级",required = false) @RequestParam(required = false, defaultValue = "1") Integer categoryLevel,
            @Parameter(description = "分类名称",required = false) @RequestParam(required = false) String name,
            @Parameter(description = "状态(true：显示 false:隐藏",required = false) @RequestParam(required = false) Boolean status,
            HttpServletRequest request
    ) throws RemoteServiceException {
        List<GoodsCategoryBean> list = goodsCategoryLogic.list(categoryLevel,name,status,request);
        return ResponseHelper.success(list);
    }

    @PostMapping("/add")
    @Operation(summary = "新增分类")
    public ResponseHelper add(
            @Parameter(description = "object",required = true) @RequestBody GoodsCategoryBean goodsCategoryBean,HttpServletRequest request) {
        goodsCategoryLogic.add(goodsCategoryBean,request);
        return ResponseHelper.success();
    }

    @PostMapping("/update")
    @Operation(summary = "修改分类")
    public ResponseHelper update(
            @Parameter(description = "object",required = true) @RequestBody GoodsCategory goodsCategory) {
        goodsCategoryLogic.update(goodsCategory);
        return ResponseHelper.success();
    }

    @PostMapping("/delete")
    @Operation(summary = "删除分类")
    public ResponseHelper delete(
            @Parameter(description = "分类id",required = true) @RequestParam String categoryId
    ) {
        goodsCategoryLogic.delete(categoryId);
        return ResponseHelper.success();
    }

    @GetMapping("/detail")
    @Operation(summary = "详情")
    public ResponseHelper<GoodsCategory> detail(
            @Parameter(description = "分类id",required = true) @RequestParam String categoryId
    ){
        GoodsCategory detail = goodsCategoryLogic.detail(categoryId);
        return ResponseHelper.success(detail);
    }
}
