package com.gree.mall.contest.controller.pc.order;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.order.OrderRefundDetailBean;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.order.OrderRefundLogic;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Slf4j
@RestController
@Tag(name = "订单售后管理", description = "订单售后管理")
@RequestMapping(value = "/pc/order/refund", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class OrderRefundController {

    private final OrderRefundLogic orderRefundLogic;

    @GetMapping("/list")
    @Operation(summary = "列表")
    public ResponseHelper<Page<OrderRefundDetailBean>> list(HttpServletRequest request,
                                                            @Parameter(description = "订单号",required = false) @RequestParam(required = false) String orderId,
                                                            @Parameter(description = "售后订单号",required = false) @RequestParam(required = false) String orderRefundId,
                                                            @Parameter(description = "用户名",required = false) @RequestParam(required = false) String userName,
                                                            @Parameter(description = "手机号",required = false) @RequestParam(required = false) String phone,
                                                            @Parameter(description = "订单状态 DSJCL=待商家处理  DSJFH=待商家发货 DMJCL=待买家处理 OVER=已完成",required = false) @RequestParam(required = false) String orderStatus,
                                                            @Parameter(description = "开始申请时间",required = false) @RequestParam(required = false) String startCreateTime,
                                                            @Parameter(description = "结束申请时间",required = false) @RequestParam(required = false) String endCreateTime,
                                                            @Parameter(description = "开始处理时间",required = false) @RequestParam(required = false) String startExamineTime,
                                                            @Parameter(description = "结束处理时间",required = false) @RequestParam(required = false) String endExamineTime,
                                                            @Parameter(description = "页号",required = true) @RequestParam(required = true) Integer pageNum,
                                                            @Parameter(description = "页大小",required = true) @RequestParam(required = true) Integer pageSize
    ) throws RemoteServiceException {
        IPage<OrderRefundDetailBean> list = orderRefundLogic
                .list(request,orderId, orderRefundId, userName, phone, orderStatus, startCreateTime, endCreateTime, startExamineTime, endExamineTime, pageNum, pageSize);
        return ResponseHelper.success(list);
    }

    @GetMapping("/refund/detail")
    @Operation(summary = "售后订单-订单退款详情")
    public ResponseHelper<OrderRefundDetailBean> refundDetail(
            @Parameter(description = "售后订单id",required = true) @RequestParam String orderRefundId
    ){
        OrderRefundDetailBean orderRefundDetailBean = orderRefundLogic.orderRefundDetail(orderRefundId);
        return ResponseHelper.success(orderRefundDetailBean);
    }

    @GetMapping("/expressInfo/select")
    @Operation(summary = "售后订单-查看物流")
    public ResponseHelper queryExpressInfo(
            @Parameter(description = "物流单号", required = true) @RequestParam(required = true) String logisticsNo
    ) throws RemoteServiceException {
        orderRefundLogic.queryExpressInfo(logisticsNo);
        return ResponseHelper.success();
    }


    @PostMapping("/examine")
    @Operation(summary = "审核")
    public ResponseHelper examine(
            @Parameter(description = "售后单号",required = true) @RequestParam(required = true) String orderRefundId,
            @Parameter(description = "OK=同意  FAIL=不同意",required = true) @RequestParam(required = true) String examineStatus,
            @Parameter(description = "卖家留言",required = false) @RequestParam(required = false) String refundMessage,
            @Parameter(description = "退款密码") @RequestParam(required = false) String refundPassword
    ) throws Exception {
        orderRefundLogic.examine(orderRefundId, examineStatus, refundMessage, refundPassword);
        return ResponseHelper.success();
    }

    @PostMapping("/ack")
    @Operation(summary = "商家确认收货")
    public ResponseHelper examine(
            @Parameter(description = "售后单号",required = true) @RequestParam(required = true) String orderRefundId
    ) throws Exception {
        orderRefundLogic.ack(orderRefundId);
        return ResponseHelper.success();
    }


    @PostMapping("/remark")
    @Operation(summary = "备注")
    public ResponseHelper remark(
            @Parameter(description = "售后单号",required = true) @RequestParam(required = true) String orderRefundId,
            @Parameter(description = "备注",required = true) @RequestParam(required = true) String remark
    ){
        orderRefundLogic.remark(orderRefundId,remark);
        return ResponseHelper.success();
    }




    @GetMapping("/downLoadExcel")
    @Operation(summary = "导出售后报表")
    public void downLoadExcel(HttpServletRequest request, HttpServletResponse response,
                                         @Parameter(description = "订单号",required = false) @RequestParam(required = false) String orderId,
                                         @Parameter(description = "售后订单号",required = false) @RequestParam(required = false) String orderRefundId,
                                         @Parameter(description = "用户名",required = false) @RequestParam(required = false) String userName,
                                         @Parameter(description = "手机号",required = false) @RequestParam(required = false) String phone,
                                         @Parameter(description = "订单状态 DSJCL=待商家处理  DSJFH=待商家发货 DMJCL=待买家处理 OVER=已完成",required = false) @RequestParam(required = false) String orderStatus,
                                         @Parameter(description = "开始申请时间",required = false) @RequestParam(required = false) String startCreateTime,
                                         @Parameter(description = "结束申请时间",required = false) @RequestParam(required = false) String endCreateTime,
                                         @Parameter(description = "开始处理时间",required = false) @RequestParam(required = false) String startExamineTime,
                                         @Parameter(description = "结束处理时间",required = false) @RequestParam(required = false) String endExamineTime
    ) throws Exception {
        orderRefundLogic.downLoadOrderRefundExcel(request,response,orderId,orderRefundId,userName,phone,orderStatus,startCreateTime,endCreateTime,startExamineTime,endExamineTime);
    }

}
