package com.gree.mall.contest.logic;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.CarouselMap;
import com.gree.mall.contest.plus.entity.Goods;
import com.gree.mall.contest.plus.service.CarouselMapService;
import com.gree.mall.contest.plus.service.GoodsService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class CarouselMapLogic {

    private final CarouselMapService carouselMapService;
    private final GoodsService goodsService;
    private final CommonLogic commonLogic;

    /**
     * 轮播图列表分页
     */
    public IPage<CarouselMap> page(HttpServletRequest request, Boolean state, Integer pageNum, Integer pageSize) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return carouselMapService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(companyWechatIds), CarouselMap::getCompanyWechatId, companyWechatIds)
                .eq(state != null, CarouselMap::getState, state)
                .orderByDesc(CarouselMap::getCreateTime)
                .page(new Page<>(pageNum, pageSize));
    }
    /**
     * 轮播图详情
     */
    public CarouselMap detail(String id){
        return carouselMapService.getById(id);
    }



    /**
     * 新增轮播图
     */
    @Transactional(rollbackFor = Exception.class)
    public void add(HttpServletRequest request, CarouselMap carouselMap) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        //新增轮播
        carouselMap.setCreateTime(new Date());
        carouselMap.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
        carouselMap.setCompanyName(adminUser.getLoginCompanyName());
        carouselMapService.save(carouselMap);
    }

    /**
     * 修改轮播图
     */
    @Transactional(rollbackFor = Exception.class)
    public void update(CarouselMap carouselMap) {
        //修改轮播
        carouselMapService.updateById(carouselMap);
    }

    /**
     * 删除轮播图
     */
    @Transactional(rollbackFor = Exception.class)
    public void delete(String carouselMapId) {
        carouselMapService.removeById(carouselMapId);
    }

    /**
     * 批量删除轮播图
     */
    @Transactional(rollbackFor = Exception.class)
    public void delete(List<String> carouselMapIds) {
        carouselMapService.removeByIds(carouselMapIds);
    }


    /**
     * 更改轮播图状态
     */
    public void updateState(String carouselMapId, Boolean state) {
        carouselMapService.lambdaUpdate().set(CarouselMap::getState, state)
                .eq(CarouselMap::getId, carouselMapId).update();
    }

    /**
     * 获取商品列表
     * @param goodsId
     * @param goodsName
     * @param pageNum
     * @param pageSize
     * @return
     */
    public IPage<Goods> getGoods(HttpServletRequest request, String goodsId, String goodsName, Integer pageNum, Integer pageSize) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        if (StringUtils.isNotBlank(goodsId)) {
            return goodsService.lambdaQuery()
                    .eq(Goods::getGoodsId, goodsId)
                    .in(CollectionUtil.isNotEmpty(companyWechatIds), Goods::getCompanyWechatId, companyWechatIds)
                    .page(new Page<>(pageNum, pageSize));
        }
        return goodsService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(companyWechatIds), Goods::getCompanyWechatId, companyWechatIds)
                .eq(Goods::getStatus, true)
                .like(StringUtils.isNotBlank(goodsName), Goods::getGoodsName, goodsName)
                .page(new Page<>(pageNum, pageSize));
    }
}
