package com.gree.mall.contest.logic.admin;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.admin.MerchantWebsitTree;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class AdminWebsitLogic {

    private final AdminWebsitService adminWebsitService;
    private final MerchantWebsitService merchantWebsitService;
    private final MerchantWebsitRegionService merchantWebsitRegionService;
    private final AdminUserService adminUserService;
    private final CommonLogic commonLogic;

    private final AdminRoleService adminRoleService;

    /**
     * 部门树
     */
    public List<MerchantWebsitTree> tree(HttpServletRequest request, Boolean isRole){
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        List<AdminRole> adminRoles = adminRoleService.lambdaQuery().in(CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds())
                ,AdminRole::getServiceProviderId,adminUser.getAdminWebsitIds()
        )
                .list();
        List<String> websids = adminRoles.stream().map(AdminRole::getAdminWebsitId).collect(Collectors.toList());


        if (isRole){
            if (CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds()))
                websids.addAll(adminUser.getAdminWebsitIds());
            List<MerchantWebsit> list = merchantWebsitService.lambdaQuery().in(MerchantWebsit::getWebsitId,websids)
                    .orderByAsc(MerchantWebsit::getCreateTime)
                    .list();

            List<MerchantWebsitTree> merchantWebsitTrees = BeanUtil.copyToList(list, MerchantWebsitTree.class);

            return merchantWebsitTrees;
        }

        List<MerchantWebsit> list = merchantWebsitService.list();
        List<MerchantWebsitTree> trees = new ArrayList<>();


        for(MerchantWebsit merchantWebsit : list){
            MerchantWebsitTree merchantWebsitTree = new MerchantWebsitTree();
            BeanUtils.copyProperties(merchantWebsit, merchantWebsitTree);
            trees.add(merchantWebsitTree);
        }
        List<MerchantWebsitTree> parentList = trees.stream()
                .filter(x ->  x.getWebsitId().equals(adminUser.getAdminWebsitId())).collect(Collectors.toList());

        for(MerchantWebsitTree adminWebsit:parentList){
            adminWebsit.setChildren(this.treeModule(trees,adminWebsit.getWebsitId()));
        }
        return parentList;
    }


    public List<MerchantWebsit> list(Boolean isAll, Boolean status){
        AdminUserCom adminUser = commonLogic.getAdminUser();

        List<MerchantWebsit> list = merchantWebsitService.lambdaQuery()
                .eq(status != null, MerchantWebsit::getStatus, status)
                .in(CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds()) && (isAll == null || !isAll), MerchantWebsit::getWebsitId,adminUser.getAdminWebsitIds())
                .list();
        return list;
    }


    /**
     *  新增部门
     */
    @Transactional
    public void add(MerchantWebsit merchantWebsit, HttpServletRequest request) throws WxErrorException, RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        Long count = merchantWebsitService.lambdaQuery().eq(MerchantWebsit::getName, merchantWebsit.getName())
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称已存在");
        }
        MerchantWebsit parent = merchantWebsitService.getById(merchantWebsit.getParentId());
        if(parent == null){
            throw new RemoteServiceException("请选择父部门");
        }
        List<MerchantWebsit> list = new ArrayList<>();
        list.addAll(commonLogic.queryAllChild(list, merchantWebsit.getWebsitId()));
        List<String> childWebsitIds = list.stream().map(MerchantWebsit::getWebsitId).collect(Collectors.toList());
        if(childWebsitIds.contains(merchantWebsit.getParentId())){
            throw new RemoteServiceException("父级部门不可以是自己或者自己的下级部门");
        }
        merchantWebsit.setType(parent.getType());
        merchantWebsit.setWebsitId(IdWorker.getIdStr());
        merchantWebsitService.save(merchantWebsit);
    }
    /**
     * 编辑部门
     */
    @Transactional
    public void update(MerchantWebsit merchantWebsit, HttpServletRequest request) throws RemoteServiceException, WxErrorException {
        MerchantWebsit old = merchantWebsitService.getById(merchantWebsit.getWebsitId());
        if(Constant.SYS_WEBSIT_NAME.contains(old.getName())){
            throw new RemoteServiceException("不可操作当前部门信息");
        }
        Long count = merchantWebsitService.lambdaQuery()
                .ne(MerchantWebsit::getWebsitId, merchantWebsit.getWebsitId())
                .eq(MerchantWebsit::getName, merchantWebsit.getName())
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称或部门编号已存在");
        }
        if(merchantWebsit.getParentId().equals("0")){
            throw new RemoteServiceException("请选择父部门");
        }
        MerchantWebsit parent = merchantWebsitService.getById(merchantWebsit.getParentId());
        if(parent == null){
            throw new RemoteServiceException("请选择父部门");
        }
        if(StringUtils.equals(merchantWebsit.getWebsitId(), merchantWebsit.getParentId())){
            throw new RemoteServiceException("父级部门不可以是自己");
        }
        List<MerchantWebsit> list = new ArrayList<>();
        list.addAll(commonLogic.queryAllChild(list, merchantWebsit.getWebsitId()));
        List<String> childWebsitIds = list.stream().map(MerchantWebsit::getWebsitId).collect(Collectors.toList());
        if(childWebsitIds.contains(merchantWebsit.getParentId())){
            throw new RemoteServiceException("父级部门不可以是自己或者自己的下级部门");
        }
        merchantWebsit.setType(parent.getType());
        merchantWebsitService.saveOrUpdate(merchantWebsit);
        //workWechatLogic.updateOrganization(merchantWebsit.getWebsitId(),merchantWebsit.getName(),merchantWebsit.getParentId(),adminUser.getAdminCompanyWechat());
    }

    /**
     * 删除部门
     */
    @Transactional
    public void delete(String websitId,HttpServletRequest request) throws WxErrorException {
        MerchantWebsit old = merchantWebsitService.getById(websitId);
        if(Constant.SYS_WEBSIT_NAME.contains(old.getName())){
            throw new RemoteServiceException("不可操作当前部门信息");
        }
        Long count = merchantWebsitService.lambdaQuery().eq(MerchantWebsit::getParentId, websitId).count();
        if(count > 0){
            throw new RemoteServiceException("当前部门存在子级，删除失败");
        }
        Long count1 = adminUserService.lambdaQuery().eq(AdminUser::getAdminWebsitId, websitId).count();
        if(count1 > 0){
            throw new RemoteServiceException("当前部门存在账号，删除失败");
        }
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        //递归删除
        List<MerchantWebsit> list = new ArrayList<>();
        list.addAll(commonLogic.queryAllChild(list,websitId));
        List<String> ids = list.stream().map(MerchantWebsit::getWebsitId).collect(Collectors.toList());
        ids.add(websitId);
        if(ids.size() > 0) {
//            for(String idd : ids){
//                workWechatLogic.deleteOrganization(idd,adminUser.getAdminCompanyWechat());
//            }
            merchantWebsitService.lambdaUpdate().in(MerchantWebsit::getWebsitId, ids).remove();
        }
    }

    /**
     * 详情
     * @param id
     */
    public MerchantWebsit detail(String id){
        MerchantWebsit merchantWebsit = merchantWebsitService.getById(id);
        return merchantWebsit;
    }

    /**
     * 递归组装数据
     * @param list
     * @param parentId
     * @return
     */
    private List<MerchantWebsitTree> treeModule(List<MerchantWebsitTree> list, String parentId){
        //最父级资源树
        if(parentId == null || parentId.equals("0")) {
            return null;
        }
        List<MerchantWebsitTree> collect = list.stream()
                .filter(e -> e.getParentId().equals(parentId)).collect(Collectors.toList());
        if(collect.size() == 0){
            return null;
        }
        for(MerchantWebsitTree bean : collect){
            bean.setChildren(this.treeModule(list, bean.getWebsitId()));
        }
        return collect;
    }



    /**
     * 保存部门的服务区域
     * @param merchantWebsitRegions
     */
    @Transactional
    public void saveRegion(List<MerchantWebsitRegion> merchantWebsitRegions){
        List<String> adminWebsitIds = merchantWebsitRegions.stream().map(MerchantWebsitRegion::getAdminWebsitId).distinct().collect(Collectors.toList());
        if(adminWebsitIds.size() != 1){
            throw new RemoteServiceException("参数异常，请检查所有部门id是否一致");
        }
//        if(merchantWebsitRegions.size() > 1){
//            throw new RemoteServiceException("暂只支持1个个保存");
//        }
        for(MerchantWebsitRegion merchantWebsitRegion : merchantWebsitRegions) {
            Long count = merchantWebsitRegionService.lambdaQuery()
                    .eq(MerchantWebsitRegion::getAdminWebsitId, merchantWebsitRegion.getAdminWebsitId())
                    .eq(MerchantWebsitRegion::getServiceCategoryId, merchantWebsitRegion.getServiceCategoryId())
                    .eq(MerchantWebsitRegion::getProvinceId, merchantWebsitRegion.getProvinceId())
                    .eq(MerchantWebsitRegion::getCityId, merchantWebsitRegion.getCityId())
                    .eq(MerchantWebsitRegion::getAreaId, merchantWebsitRegion.getAreaId())
                    .eq(MerchantWebsitRegion::getStreetId, merchantWebsitRegion.getStreetId())
                    .count();
            String pcas = merchantWebsitRegion.getProvince() + merchantWebsitRegion.getCity() + merchantWebsitRegion.getArea() + merchantWebsitRegion.getStreet();
            if (count > 0) {
                throw new RemoteServiceException(pcas + "已存在，请勿重复添加");
            }
            merchantWebsitRegion.setPcas(pcas);
            merchantWebsitRegionService.save(merchantWebsitRegion);
        }
        //刷新服务商的服务类目冗余字段
        reloadServiceCategoryName(merchantWebsitRegions.get(0).getAdminWebsitId());
    }

    /**
     * 部门的服务区域列表
     * @param adminWebsitId
     * @return
     */
    public List<MerchantWebsitRegion> listRegion(String adminWebsitId, String keyword, String serviceCategoryId){
        List<MerchantWebsitRegion> list = merchantWebsitRegionService.lambdaQuery()
                .like(StringUtils.isNotBlank(keyword), MerchantWebsitRegion::getPcas,keyword)
                .eq(StringUtils.isNotBlank(serviceCategoryId), MerchantWebsitRegion::getServiceCategoryId,serviceCategoryId)
                .eq(MerchantWebsitRegion::getAdminWebsitId, adminWebsitId).list();
        return list;
    }

    /**
     * 删除服务区域
     */
    public void deleteRegion(String id){
        MerchantWebsitRegion merchantWebsitRegion = merchantWebsitRegionService.getById(id);
        merchantWebsitRegionService.removeById(id);
        //刷新服务商的服务类目冗余字段
        reloadServiceCategoryName(merchantWebsitRegion.getAdminWebsitId());
    }


    /**
     * 刷新服务商的服务类目id
     * @param adminWebsitId
     * @return
     */
    public void reloadServiceCategoryName(String adminWebsitId){
        String serviceCategoryName = merchantWebsitRegionService.lambdaQuery().eq(MerchantWebsitRegion::getAdminWebsitId, adminWebsitId)
                .list().stream()
                .map(MerchantWebsitRegion::getServiceCategoryName)
                .distinct()
                .collect(Collectors.joining());

//        serviceProviderService.lambdaUpdate()
//                .set(ServiceProvider::getServiceCategoryName,serviceCategoryName)
//                .eq(ServiceProvider::getId,adminWebsitId).update();
    }








}
