package com.gree.mall.contest.logic.common;

import com.gree.mall.contest.enums.admin.UpdateLogRemarkEnum;
import com.gree.mall.contest.plus.entity.UpdateLog;
import com.gree.mall.contest.plus.service.UpdateLogService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 修改日志
 * @author ：lijh
 * @description：TODO
 * @date ：2024/3/11 15:33
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class UpdateLogLogic {

    private final UpdateLogService updateLogService;

    /**
     * 新增修改日志
     * @param objId 各业务数据主键id
     * @param type 操作类型(例如：更改公司名称)
     * @param content  操作内容(例如：从 广州众炬 修改为 广州格力)
     * @param updateLogRemarkEnum 备注业务模块，自行增加枚举类型
     */
    public void add(String objId, String type, String content, UpdateLogRemarkEnum updateLogRemarkEnum){
        UpdateLog updateLog = new UpdateLog();
        updateLog.setObjId(objId);
        updateLog.setType(type);
        updateLog.setContent(content);
        updateLog.setRemark(updateLogRemarkEnum.getRemark());
        updateLog.insert();
    }

    /**
     * 修改日志列表
     * @param objId
     * @return
     */
    public List<UpdateLog> list(String objId,String remark){
       return updateLogService.lambdaQuery().eq(StringUtils.isNotEmpty(objId),UpdateLog::getObjId,objId)
               .eq(StringUtils.isNotEmpty(remark),UpdateLog::getRemark,remark)
               .orderByDesc(UpdateLog::getCreateTime).list();
    }


}
